-- Sampler (PG).lua -- VLC Extension

-- Feel free to edit/add profiles in the format:
-- {"Name of the profile", Samples, Duration, true / false for random position, Skip begin, End}
profiles={
	{"-- Profile --",0,0,false,0,0},
	{"Music collage",1,15,true,20,20},
	{"Film preview",6,20,false,240,480},
	{"Series without intro/credits",0,0,false,60,60},
}

-- default profile
dp = 1
-- presets
samples = profiles[dp][2]  -- number of samples per track
sample_duration = profiles[dp][3]  -- time in seconds
random_position = profiles[dp][4]  -- true/false
skip_begin = profiles[dp][5]  -- seconds
skip_end = profiles[dp][6]  -- seconds

id_correction=0 -- Autofeed related. Set 1 for VLC-2.0.x, 0 for VLC-2.1+

function descriptor()
	return {
		title = "Sampler (PG)";
		version = "1.0";
		author = "lubozle";
		url = 'http://addons.videolan.org/content/show.php?content=149688';
		shortdesc = "Samples playlist generator.";
		description = "<div style=\"background-color:lightgreen;\"><b>Sampler</b> is VLC extension that generates samples of playlist items of desired duration.</div>";
		capabilities = {"meta-listener"}
	}
end
function activate()
	math.randomseed(os.time())
	create_dialog()
end
function deactivate()
end
function close()
	vlc.deactivate()
end

function meta_changed()
-- Autofeed is not reliable feature!
	if checkbox_autofeed:get_checked() and current_id~=vlc.playlist.current() then
		current_id=vlc.playlist.current()
vlc.msg.info("[Sampler] current_id = " .. tostring(current_id))
		local pl = vlc.playlist.get("playlist",false).children
		local last_id=pl[#pl].id+id_correction
vlc.msg.info("[Sampler] last_id = " .. tostring(last_id))
vlc.msg.info("[Sampler] id_correction = " .. tostring(id_correction))
		if last_id==current_id then
			Feed()
		end
	end
--
end

function generate_samples()

	set_position=nil
	samples_positions={}
	if (duration - skip_begin - skip_end)>sample_duration then
		possible_samples=math.floor((duration - skip_begin - skip_end) / sample_duration)
		if samples>possible_samples then samples_number=possible_samples else samples_number=samples end
vlc.msg.info("[Sampler] samples_number = "..samples_number)
		if samples_number>1 then
			skipper = math.floor(((duration - skip_begin - skip_end) - (samples_number * sample_duration)) / (samples_number - 1))
		else
			skipper = 0
		end
vlc.msg.info("[Sampler] skipper = "..skipper)

		for i=1,samples_number do
			if random_position==true then
				set_position=skip_begin+math.floor(math.random()*(duration - skip_begin - skip_end - sample_duration))
			else
				if i==1 then
					set_position=skip_begin
				else
					set_position=set_position + sample_duration + skipper
				end
			end
vlc.msg.info("[Sampler] set_position #"..i.." = "..set_position)
			table.insert(samples_positions, set_position)
		end
vlc.msg.info("[Sampler] samples_positions = "..#samples_positions)
	elseif duration > sample_duration then
		if random_position==true then
			set_position=math.floor(math.random()*(duration - sample_duration))
			table.insert(samples_positions, set_position)
		end
	end

	if #samples_positions==0 then
		table.insert(samples_positions, 0)
	end
	
end

----------
function create_dialog()
	d = vlc.dialog("Sampler (PG)")
	button_read = d:add_button("&Read playlist", Read,1,1,1,1)
	label_items_duration = d:add_label("0/0 [00:00:00]",2,1,1,1)

	dropdown_profile = d:add_dropdown(1,3,1,1)
	for i,profile in pairs(profiles) do
		dropdown_profile:add_value(profile[1], i)
	end
	d:add_button("Apply profile", click_Profile,2,3,1,1)

	d:add_label("<b>Samples per item:</b>",1,4,1,1)
	textinput_samples = d:add_text_input(samples,2,4,1,1)
	d:add_label("<b>Sample duration [s]:</b>",1,5,1,1)
	textinput_duration = d:add_text_input(sample_duration,2,5,1,1)
	checkbox_random_position = d:add_check_box("Random position",random_position,2,6,1,1)		
	d:add_label("<b>Skip begin/end [s]:</b>",1,7,1,1)
	textinput_skip_begin = d:add_text_input(skip_begin,2,7,1,1)
	textinput_skip_end = d:add_text_input(skip_end,2,8,1,1)

	d:add_button("&Clear playlist", Clear,1,10,1,1)
	button_help = d:add_button("HELP", click_HELP,2,10,1,1)
	checkbox_shuffle = d:add_check_box("Shuffle tracks",false,1,11,1,1)		
	checkbox_autofeed = d:add_check_box("Autofeed on last track*",false,2,11,1,1)		
	d:add_button("&Feed playlist", Feed,1,12,1,1)
	label_feeds = d:add_label("0 (00:00:00)",2,12,1,1)
end
function Read()
	local pd=playlist_duration()
	label_items_duration:set_text(items_withduration.."/"..items_all.." ["..pd.."]")
end
function Clear()
	vlc.playlist.clear()
	feeds=0
	label_feeds:set_text(feeds)
end
feeds=0
function Feed()
	click_Apply()

	feeds=feeds+1
	order = {}
	for j=1,items_withduration do
		order[j] = j
	end
	if checkbox_shuffle:get_checked() then
		order = shuffle(order)
	end

	for _, item in ipairs(order) do
		local item=table_items[item]
		---
		if sample_duration==0 or samples==0 then
			local options={}
			if (item.duration-skip_begin-skip_end)>0 then
				if skip_begin>0 then table.insert(options,"start-time="..skip_begin) end
				if skip_end>0 then table.insert(options,"stop-time="..(item.duration-skip_end)) end
			end
			vlc.playlist.enqueue({{path=item.path, name=item.name, duration=item.duration, options=options}})
		else
			duration=item.duration
			generate_samples()
			for _, sample_position in ipairs(samples_positions) do
				vlc.playlist.enqueue({{path=item.path, name=item.name, duration=item.duration, options={"start-time="..sample_position, "stop-time="..(sample_position+sample_duration)}}})
			end
		end
		---
	end

	label_feeds:set_text(feeds)
end

function shuffle(t)
    local n = #t
 
    while n > 2 do
        -- n is now the last pertinent index
        local m = math.random(n) -- 1 <= m <= n
        -- Quick swap
        t[n], t[m] = t[m], t[n]
        n = n - 1
    end
 
    return t
end

function dur_to_time(duration)
	if duration>0 then
		local durationHour = math.floor(duration / 3600)
		local durationMinute = math.floor((duration % 3600) / 60)
		local durationSecond = math.floor(duration % 60)
		return durationHour, durationMinute, durationSecond
	else
		return 0, 0, 0
	end
end
function playlist_duration()
	items_all = 0
	items_withduration = 0
	local sum = 0
	local play_list = vlc.playlist.get("playlist",false)
	table_items={}
	output_table={}
	for k, item in pairs(play_list.children) do
		items_all = items_all + 1
		if item.duration ~= -1 then
			items_withduration = items_withduration + 1
			sum = sum + item.duration
			table.insert(table_items, {path = item.path, name = item.name, duration = item.duration})

			local h, m, s = dur_to_time(item.duration)
			table.insert(output_table, items_withduration .. ".) " .. item.name .. " [" .. tostring(string.format("%02d:%02d:%02d", h, m, s)) .. "]")
		end
	end

	local h, m, s = dur_to_time(sum)
	return tostring(string.format("%02d:%02d:%02d", h, m, s))
end
----------
function click_Profile()
	local profile = profiles[dropdown_profile:get_value()]
	textinput_samples:set_text(profile[2])
	textinput_duration:set_text(profile[3])
	checkbox_random_position:set_checked(profile[4])
	textinput_skip_begin:set_text(profile[5])
	textinput_skip_end:set_text(profile[6])
end
function click_Apply()
	samples = tonumber(textinput_samples:get_text())
	sample_duration = tonumber(textinput_duration:get_text())
	random_position = checkbox_random_position:get_checked()
	skip_begin = tonumber(textinput_skip_begin:get_text())
	skip_end = tonumber(textinput_skip_end:get_text())
	-- to test input values...
end
function click_HELP()
	local help_text=[[
<style type="text/css">
body {background-color:white;} 
.hello{font-family:"Arial black";font-size:48px;color:red;background-color:lime}
#header{background-color:lightgreen;}
.marker_red{background-color:#FF7FAA;}
.marker_green{background-color:lightgreen;}
.input{background-color:lightblue;}
.button{background-color:silver;}
.tip{background-color:#FFFF7F;}
#footer{background-color:lightblue;}
</style>

<div id=header><b>Sampler (PG)</b> is VLC Lua Extension that generates samples of playlist items of desired duration. Now you can create a music collage, automatically preview your film collection, skip intro and credits in series, ...</div>
<hr />
<center><b class=marker_red>&nbsp;Instructions&nbsp;</b></center>
Enqueue your audio/video files in the playlist. Then:<br />
<b class=marker_red>1.)</b> Press <b class=button>[ Read playlist ]</b> button to read all playlist tracks with available duration. You will see total tracks and time information next to the button.<br />
<b class=marker_red>2.)</b> Select a desired <b class=input>profile</b> in the drop-down menu and load it by pressing <b class=button>[ Apply profile ]</b> button. All fields will be updated with desired presets.<br />
Feel free to add your own profiles by editing the Lua script in a text editor.<br />
<b class=marker_red>3.)</b> You can set your own values if you are not happy with predefined ones.<br />
<b class=input>Samples per item:</b> how many samples you want to be generated for each playlist item.<br />
<b class=input>Sample duration [s]:</b> in seconds, how long each sample should be.<br />
<b class=input>[ ] random position</b> - tick it if you want samples to have random position rather than ordered sequentionally.<br />
<b class=input>Skip begin/end [s]:</b> - in seconds, how long part of each playlist item should be ignored at the beginning and at the end of it (intro and credits).<br />
<b class=marker_red>4.)</b> Press <b class=button>[ Clear playlist ]</b> button to prepare empty playlist for generated tracks.<br />
<b class=marker_red>5.)</b> Tick <b class=input>[ ] Shuffle track</b> if you want tracks to be shuffled. Tick <b class=input>[ ] Autofeed on last track</b> if you want to automatically generate and enqueue a new batch of tracks when VLC starts to play the last track in the playlist. *The "Autofeed" feature is not reliable on my system (Windows XP).<br />
<b class=marker_red>6.)</b> Finally press <b class=button>[ Feed playlist ]</b> button to generate a batch of tracks in playlist. The generated tracks contain "start-time", "stop-time" VLC options. Press the button again to generate and enqueue another batch.<br />
Start to play.<br />
<br />
Depending on used values and duration of playlist item, algorithm parses samples in several ways, schematicly:
<pre>|-SB-+&lt;------S0------>+-SE-|
|-SB-+--+S2S1+---+S3+-+-SE-|
|-SB-+S1+---+S2+---+S3+-SE-|
|---+S1+-|
|S1+-----|
|S1|</pre>
<div class=tip>Show/hide VLC playlist by pressing the "Show playlist" button in VLC control panel or go in VLC menu:<br />
<b>View > Playlist</b><br />
You can save the generated playlist in VLC menu:<br />
<b>Media > Save Playlist to File...</b></div>
<hr />
<div id=footer>
<b>Homepage:</b> <a href="http://addons.videolan.org/content/show.php?content=149688">Sampler</a><br />
<b>Forum:</b> <a href="http://forum.videolan.org/viewtopic.php?f=29&t=99501">VLC extension: Sampler</a><br />
Please, visit us and bring some new ideas.<br />
Learn how to write your own VLC Lua scripts and share them with us.<br />
Help to build happy VLC community :o)</div>
]]
	d:del_widget(button_help)
	html_help = d:add_html(help_text,1,15,2,1)
	button_helpx = d:add_button("HELP [x]", click_HELPx,2,10,1,1)
	d:update()
end
function click_HELPx()
	d:del_widget(html_help)
	d:del_widget(button_helpx)
	html_help=nil
	button_helpx=nil
	button_help = d:add_button("HELP", click_HELP,2,10,1,1)
	d:update()
end